/****************************************************************************
 * This file is part of PaToH Matlab Interface
 * Copyright (c) 2009 Bora Ucar and Umit V. Catalyurek
 * ----------------------------------------------------------------------
 * PaToH Matlab Interface is released under the GNU Lesser General Public
 * License. A copy of the license is included in this distribution in
 * COPYING.LESSER file.
 * ----------------------------------------------------------------------
 * $Id$
 *
 * Matlab CMEX-file
 * Developed by B. Ucar and U. V. Catalyurek
 *
 * Usage:
 * [partv, ptime] = PaToH(hyp, K [,nconst, cw, nw, imbal, cutdef])
 *
 *
 * Input Arguments:
 * hyg    : hypergraph given as matrix: hyperedges/nets are columns,
 *          vertices/cells are rows.
 * K      : numparts
 * nconst : number of cell weight constraints (optional)
 * cw     : cell weights (optional)
 * nw     : net weights  (optional)
 * imbal  : allowed imbalance ratio (optional)
 * cutdef : cut function definition ('CON' or 'CUT' optional). CON is
 *          default
 *
 * Output Arguments:
 *
 * partv  : 0-based part numbers for each vertex/cell
 * ptime  : time spent in the wrappers around PaToH and PaToH itself.
 *
 *****************************************************************************
 * How to build:
 *
 *  in matlab run:
 *
 *   mex PaToH.c -lpatoh -LPATOHLIBDIR/ -IPATOH_H_DIR -lm
 *
 * On a 64-bit machine add: "-largeArrayDims"
 *
 * If you would like to customize the build process use "-f" option of mex.
 * For example with if you have your own "mexopts.sh" file (a sample file
 * is listed below) in the same directory you can build with the command:
 *
 *   mex -f mexopts.sh -v PaToH.c -lpatoh -LPATOHLIBDIR/ -IPATOH_H_DIR -lm
 *
 *****************************************************************************
 * My mexops.sh file looks like for option glnx86
 * #----------------------------------------------------------------------------
 * glnx86)
 * #----------------------------------------------------------------------------
 * RPATH="-Wl,--rpath-link,$TMW_ROOT/bin/$Arch"
 * #           gcc -v
 * #           gcc version 3.2.3
 * CC='gcc'
 * CFLAGS='-fPIC -ansi -D_GNU_SOURCE -pthread -fexceptions -Wall'
 * CLIBS="$RPATH $MLIBS -lm -lstdc++"
 * COPTIMFLAGS='-O -DNDEBUG'
 * CDEBUGFLAGS='-g'
 * #
 * #           g++ -v
 * #           gcc version 3.2.3
 * CXX='g++'
 * CXXFLAGS='-fPIC -ansi -D_GNU_SOURCE -pthread '
 * CXXLIBS="$RPATH $MLIBS -lm"
 * CXXOPTIMFLAGS='-O -DNDEBUG'
 * CXXDEBUGFLAGS='-g'
 * #
 * #           g77 -fversion
 * #           GNU Fortran (GCC 3.2.3) 3.2.3 20030422 (release)
 * #           NOTE: g77 is not thread safe
 * FC='g77'
 * FFLAGS='-fPIC -fexceptions -Wall'
 * FLIBS="$RPATH $MLIBS -lm -lstdc++"
 * FOPTIMFLAGS='-O'
 * FDEBUGFLAGS='-g'
 * #
 * LD='gcc'
 * LDEXTENSION='.mexglx'
 * LDFLAGS="-Wall -pthread -shared -Wl,--version-script,$TMW_ROOT/extern/lib/$Arch/$MAPFILE"
 * LDOPTIMFLAGS='-O'
 * LDDEBUGFLAGS='-g'
 * #
 * POSTLINK_CMDS=':'
 * #----------------------------------------------------------------------------
 *
 *
 *****************************************************************************/
#include <math.h>
#include "mex.h"
#include "patoh.h"
#include <time.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/resource.h>


#define SETRANDSEED

#define A_IN     (prhs[0])
#define K_IN     (prhs[1])

#define NCONST_IN (prhs[2])

#define CWGHTS_IN (prhs[3])
#define NWGHTS_IN (prhs[4])
#define IMBAL_IN  (prhs[5])
#define CUTDEF_IN (prhs[6])

#define PARTV_OUT (plhs[0])
#define PTIME_OUT (plhs[1])


/*Caution should delete these lines for matlab version > 7.1*/
/*
 * typedef int mwIndex;
 * typedef int mwSize;
 */

static void wrapPaToH(int _c, int _n,
        int *xpins, int *pins,
        int *cwghts, int *nwghts,
        int _k, int _nconst,
        int *partvec,
        int *partwghts, double imbal, int cuttype);


static void setVal(int *lst, int sz, int ival) {
    int i;
    for (i=0;i < sz; i++)
        lst[i] = ival;
}

static double u_wseconds(void) {
    struct timeval tp;
    
    gettimeofday(&tp, NULL);
    
    return (double) tp.tv_sec + (double) tp.tv_usec / 1000000.0;
}

void mexFunction(int nlhs, mxArray *plhs[ ],
        int nrhs, const mxArray *prhs[ ]) {
    
    
    int i;
    mwSize m, n, ne;
    /*PaToH's arguments*/
    int _k;
    int _nconst;
    int *cwghts;
    int *nwghts;
    int *partvec;
    int *partwghts;
    double imbal;
    int *xpins, *pins;
    int _c, _n;
    /*pointers from matlab call*/
    mwIndex *irn_in;
    mwIndex *jcn_in;
    double *k_in_pr;
    double *nconst_pr;
    double *imbal_pr;
    double *cwghts_pr = NULL;
    double *nwghts_pr = NULL;
    char *cutdef ;
    
    
    double *partv_pr;
    double *ptime_pr;
    
    double elaptime;
    char   errmsg[256];
    
    int cuttype;
    
    if ((nrhs < 2) || (nrhs > 7) || !((nlhs == 1) ||(nlhs ==2)))
        mexErrMsgTxt("[cellpartv,time] = PaToH(hyp, K [,nconst, cwghts, nwghts, imbal, cutdef])");
    
    /* check A */
    if ((mxGetNumberOfDimensions(A_IN)!=2) || (!mxIsSparse(A_IN)) )
        mexErrMsgTxt("First parameter must be a sparse matrix");
    
    _n = n = mxGetN(A_IN); /* number of columns / nets */
    _c = m = mxGetM(A_IN); /* number of rows / cells (vertices) */
    
    k_in_pr = mxGetPr(K_IN);
    _k = (int)ceil(k_in_pr[0]);
    
    if (nrhs>2) {
        nconst_pr = mxGetPr(NCONST_IN);
        _nconst = (int) ceil( nconst_pr[0]);
    } else
        _nconst = 1;
    
    /* check cell weights */
    if (nrhs>3) {
        mwSize wm, wn;
        
        if (mxIsSparse(CWGHTS_IN))
            mexErrMsgTxt("Cell weights should be dense vector");
        
        wn = mxGetN(CWGHTS_IN); /* number of columns */
        wm = mxGetM(CWGHTS_IN); /* number of rows */
        
        if (!(((wn==1) && (wm== _nconst * _c)) || ((wm==1) && (wn== _nconst * _c)))) {
            snprintf(errmsg, sizeof(errmsg), "Cell weights is (size of %ld by %ld) should be a row or column vector of size #Constraints x #Cells (%d x %d)", wm, wn, _nconst, _c);
            mexErrMsgTxt(errmsg);
        }
        
        cwghts_pr = mxGetPr(CWGHTS_IN);
    }
    
    /* check net weights */
    if (nrhs>4) {
        mwSize wm, wn;
        
        if (mxIsSparse(NWGHTS_IN))
            mexErrMsgTxt("Net weights should be dense vector");
        
        wn = mxGetN(NWGHTS_IN); /* number of columns */
        wm = mxGetM(NWGHTS_IN); /* number of rows */
        
        if ( !(((wn==1) && (wm==_n)) || ((wm==1) && (wn==_n)))) {
            snprintf(errmsg, sizeof(errmsg), "Net weights is (size of %ld by %ld) should be a row or column vector of size #Nets (%d)", wm, wn, _n);
            mexErrMsgTxt(errmsg);
        }
        
        nwghts_pr = mxGetPr(NWGHTS_IN);
    }
    
    /* set imbalance */
    if(nrhs > 5){
        imbal_pr = mxGetPr(IMBAL_IN);
        imbal = imbal_pr[0];
    } else
        imbal = 0.03;
    if(nrhs == 7){
        if ( mxIsChar(CUTDEF_IN) != 1)
            mexErrMsgTxt("cutdef must be a string.");
        cutdef  = mxArrayToString(CUTDEF_IN);
    }else {
        cutdef = mxCalloc(5, sizeof(char));
        sprintf(cutdef, "%s", "CON");
    }
    
    jcn_in = mxGetJc(A_IN);
    irn_in = mxGetIr(A_IN);
    ne = jcn_in[n];
    
    xpins = (int*) mxCalloc(_n+1, sizeof(int));
    pins = (int*) mxCalloc(ne, sizeof(int));
    
    for (i=0; i<= n; i++)
        xpins[i] = jcn_in[i];
    for (i=0; i<ne; i++)
        pins[i] = irn_in[i] ;
    
    cwghts = (int*) mxCalloc(_c*_nconst, sizeof(int));
    nwghts = (int*) mxCalloc(_n, sizeof(int));
    if (!cwghts || !nwghts)
        mexErrMsgTxt("PaToH mex error in allocating memory for vertex and net weights");
    
    if (cwghts_pr) {
        for(i=0; i< _c*_nconst; i++)
            cwghts[i] = (int)ceil(cwghts_pr[i]);
    } else
        setVal(cwghts, _c, 1);
    
    if (nwghts_pr) {
        for(i=0; i<_n; i++)
            nwghts[i] = (int)ceil(nwghts_pr[i]);
    } else
        setVal(nwghts, _n, 1);
    
    
    partvec = (int*) mxCalloc(_c, sizeof(int));
    partwghts = (int*) mxCalloc(_k*_nconst, sizeof(int));
    
    elaptime = -u_wseconds();

    if( (toupper( *cutdef ) == 'C' ) && (toupper(*(cutdef + 1)) == 'O' )
                                     && (toupper(*(cutdef + 2))  == 'N' ))
        cuttype = PATOH_CONPART;
    else if((toupper(*cutdef) == 'C' ) && (toupper(*(cutdef + 1)) == 'U' )
                                       && (toupper(*(cutdef + 2)) == 'T' ))
        cuttype = PATOH_CUTPART;
    else

        mexErrMsgTxt("cuttype should be either CON or CUT");
    wrapPaToH(_c,  _n,
            xpins , pins,
            cwghts, nwghts,
            _k, _nconst,
            partvec,
            partwghts, imbal, cuttype);
    
    elaptime += u_wseconds();
    
    /* Create an mxArray for the output */
    PARTV_OUT = mxCreateDoubleMatrix(m, 1, mxREAL);
    partv_pr  = mxGetPr( PARTV_OUT );
    for (i=0; i < _c; i++)
        partv_pr [ i ] = partvec[i];
    if (nlhs == 2) {
        PTIME_OUT = mxCreateDoubleMatrix(1, 1, mxREAL);
        ptime_pr = mxGetPr(  PTIME_OUT );
        ptime_pr[0] =  elaptime;
    }
    
    mxFree(cwghts);
    mxFree(nwghts);
    mxFree(xpins);
    mxFree(pins);
    mxFree(partvec);
    mxFree(partwghts);
}


static void wrapPaToH(int _c, int _n,
        int *xpins, int *pins,
        int *cwghts, int *nwghts,
        int _k, int _nconst,
        int *partvec,
        int *partwghts, double imbal, int cuttype) {
    int cut;
    PaToH_Parameters       args;
#ifdef SETRANDSEED
static int patohseed=-1;

if (patohseed==-1) {
    struct timeval tp;
    
    gettimeofday(&tp, NULL);
    
    patohseed = tp.tv_sec;
} else
    ++patohseed;
#endif

PaToH_Initialize_Parameters(&args, cuttype,
        PATOH_SUGPARAM_DEFAULT);
args._k = _k;

if (PaToH_Alloc(&args, _c, _n, _nconst, cwghts, nwghts,
        xpins, pins))
    mexErrMsgTxt("PaToH mex error in allocating memory for PaToH");

#ifdef SETRANDSEED
args.seed = patohseed;
#endif
args.final_imbal = imbal;
args.init_imbal = imbal * 2.0;
/*if (args.init_imbal > 0.2)
    args.init_imbal = 0.20;
*/
PaToH_Part(&args, _c, _n, _nconst, 0, cwghts, nwghts,
           xpins, pins, NULL, partvec, partwghts, &cut);
/*
mexPrintf("PaToH cut is %d\n", cut);
*/
PaToH_Free();
}


