function [nnzpv,outpv,inpv,ptime] = PaToHMatrixPart(mat, K, dim, partitioner, imbal_in)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%[nnzpv, outpv, inpv, time] = PaToHMatrixPart(mat, K, dim, partitioner,imbal_in)
%  nnzpv : partno on nnz basis. This is a sparse matrix, with the same
%          sparsity of the input matrix mat.
%  outpv : the partvector of the output vector of spmxv with mat
%  inpv  : the partvector of the input vector of spmxv with mat
%  The part vectors contain integers from 1 to K.
%  ptime  : partitioning time
%
%  mat   : the input matrix to partition.
%  K     : the number of parts.
%  dim   : partitioning dimension in string. For now supports
%               + 'RWS' : Rowwise symmetric (vector) partitioning
%               + 'RWU' : Rowwise unsymmetric (vector) partitioning
%               + 'CWS' : Columnwise symmetric (vector) partitioning
%               + 'CWU' : Columnwise unsymmetric (vector) partitioning
%               + 'FGS' : Fine-grain symmetric (vector) partitioning
%               + 'FGU' : Fine-grain unsymmetric (vector) partitioning
%               + 'JLS' : Jagged-like symmetric (vector) partitioning
%               + 'JLU' : Jagged-like unsymmetric (vector) partitioning
%               + 'CHS' : Checkerboard-like symmetric (vector) partitioning
%               + 'CHU' : Checkerboard-like unsymmetric (vector) partitioning
%
%  partitioner: integer from 0 to 4 (see below). Default is 0---performs
%               hypergraph-based partitioning with PaToH.
%               Not all of 'dim' are implemented for partitioner>0.
%
%  imbal_in : permitted imabalance ratio. Default is 3%.
% 
%  For 2D partitioning options, K is factored as k1xk2, k2>=k1, k2 is
%         close to k1 as much as possible.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of PaToH Matlab Interface
% Copyright (c) 2009 Bora Ucar and Umit V. Catalyurek
% ----------------------------------------------------------------------
% PaToH Matlab Interface is released under the GNU Lesser General Public
% License. A copy of the license is included in this distribution in
% COPYING.LESSER file.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Date       : July 13, 2007
% Note       : added partitioner parameter
%    0 (default) use PaToH
%    1           Random
%    2           Block-row-balanced
%    3           Block-nonzero-balanced
%    4           Round-robin
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (nargin < 3)
    help PaToHMatrixPart
    error('Missing arguments: please provide minimum 3 required arguments: mat, K, dim');   
elseif (nargin == 3)
    partitioner = 0; % default is to use PaToH
end
if (nargin < 5)
    imbal = 0.03;
else
    imbal = imbal_in;
end
outpv = 0; inpv=0; nnzpv=0; ptime=0;
[m,n] = size(mat);
if (PaToHisSymmetricPart(dim) == 1)
    if (m~=n)
        error('symmetric partitioning requested with a rectangular matrix');
    end
end

if (isPart2DMesh(dim) == 1)
    [k1,k2] = meshFactor(K);
end

if (strcmpi(dim, 'RWS') ==1)
    [nnzpv,outpv,inpv,ptime] = rowwiseSymPart(mat, K, partitioner);
elseif (strcmpi(dim, 'RWU') == 1)
    [nnzpv,outpv,inpv,ptime] = rowwiseUnsymPart(mat,K, partitioner);
elseif (strcmpi(dim, 'CWS') == 1)
    [nnzpv,outpv,inpv,ptime] = colwiseSymPart(mat,K, partitioner);
elseif (strcmpi(dim, 'CWU') == 1)
    [nnzpv,outpv,inpv,ptime] = colwiseUnsymPart(mat,K, partitioner);
elseif (strcmpi(dim, 'FGS') == 1)
    [nnzpv,outpv,inpv,ptime] = fineGrainSymPart(mat,K, partitioner);
elseif (strcmpi(dim, 'FGU') == 1)
    [nnzpv,outpv,inpv,ptime] = fineGrainUnsymPart(mat,K, partitioner);
elseif (strcmpi(dim, 'JLS') == 1)
    [nnzpv, outpv, inpv,ptime] = jaggedSymPart(mat,k1,k2, partitioner);
elseif (strcmpi(dim, 'JLU') == 1)
    [nnzpv,outpv,inpv,ptime] = jaggedUnsymPart(mat,k1,k2, partitioner);
elseif (strcmpi(dim, 'CHS') == 1)
    [nnzpv,outpv,inpv,ptime] = chkbrdSymPart(mat,k1,k2, partitioner);
elseif (strcmpi(dim, 'CHU') == 1)
    [nnzpv,outpv,inpv,ptime] = chkbrdUnsymPart(mat,k1,k2, partitioner);
else
    error('third option %s is not recognized.', dim);
end

checkOutPv = find(outpv <= 0);
checkInPv = find(inpv <= 0);
if (~isempty(checkOutPv))
    error('check out pv %d', length(checkOutPv));
end
if (~isempty(checkInPv))
    error('check in pv %d\n',length(checkInPv));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [k1,k2] = meshFactor(K)
% Given K, finds integers k1 and k2 such that k1xk2 = K, k2>=k1, and k2 is
% close to k1 as much as possible.
k1 = floor(sqrt(K));
if (k1*k1 == K)
    k2 = k1;
else
    for kk=k1:-1:1
        if (mod(K, kk) == 0)
            k1 = kk;
            break;
        end
    end
    k2 = K/k1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function isit = isPart2DMesh(dim)
% Simple code fragment to reduce number of code lines somewhere else.
if ( (strcmpi(dim,'JLS') == 1) ||...
        (strcmpi(dim, 'JLU') == 1) ||...
        (strcmpi(dim, 'CHS') == 1) ||...
        (strcmpi(dim, 'CHU') == 1))
    isit=1;
else
    isit=0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [pv,ptime] = blockPart(m, K)
%Perform blockwise partitioning using natural ordering. Balance the number
%of rows per part.
stime = cputime;
pv = zeros(m,1);
n = ceil(m/K);
for i=1:m
    pv(i) = floor((i-1)/n);
end
ptime = cputime - stime;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [pv, ptime] = blockPartWithWeight(m, K, cw)
%Perform blockwise partitioning using natural ordering. Try to balance the
%number of nnz per part by assigning rows in their entireity.
stime = cputime;
avgw = sum(cw) / K;
curw = 0; part = 0;
pv = zeros(m,1);
for i=1:m
    pv(i) = part;
    curw = curw + cw(i);
    if (curw>avgw)
        part = part + 1;
        curw = 0;
    end
end
ptime = cputime - stime;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [pv,ptime] = randomPart(m, K)
%Perform random partitioning of rows.
stime = cputime;
pv =  mod([1:m]',K);
pv = pv(randperm(m));
ptime = cputime - stime;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [pv,ptime] = roundRobinPart(m, K)
%Perform roundrobin partitioning.
stime = cputime;
pv = mod([1:m]',K);
ptime = cputime-stime;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [pv,ptime] = Partition(partitioner, A, m, K, const, cw, nw)
%Wrapper for different partitioners.
if (partitioner==0)
    if (nargin==7)
        [pv,ptime] = PaToH(A, K, const, cw, nw);
    else
        [pv,ptime] = PaToH(A, K, const, cw);
    end
elseif (partitioner == 1)
    [pv,ptime]  = randomPart(m, K);
elseif (partitioner == 2)
    [pv,ptime] = blockPart(m, K);
elseif (partitioner == 3)
    [pv,ptime] = blockPartWithWeight(m, K, cw);
elseif (partitioner == 4)
    [pv,ptime] = roundRobinPart(m, K);
else
    error('unknow partitioner=%d', partitioner);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [nnzpv,outpv,inpv,ptime] = rowwiseSymPart(mat,K, partitioner)
[m,n] = size(mat);
ptime = 0;
if (K==1)
    outpv = ones(m,1);
    inpv = ones(n,1);
    nnzpv = spones(mat);
else
    A = spones(mat);
    if (n > 1)
        rowsz = A*ones(n,1);
    else
        rowsz = full(A);
    end
    In = speye(m);
    A = (A | In);

    if (sum(rowsz) ~= nnz(mat))
        error('spones yield different numnnz??? Please let Bora know.\n');
    end
    [pv,ptime] = Partition(partitioner, A, m, K, 1, rowsz);
    outpv = pv + 1;
    inpv = outpv;

    [rr, cc, pno] = find(mat);

    pno(1:length(rr)) = outpv(rr);

    nnzpv=sparse(rr,cc, pno, m, n);
    if (nnz(nnzpv)~=nnz(mat))
        error('in RWS nnz(nnzpv)~=nnz(mat)\n');
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [nnzpv,outpv,inpv,ptime] = rowwiseUnsymPart(mat,K, partitioner)
[m,n] = size(mat);
ptime= 0;
if (K==1)
    outpv = ones(m,1);
    inpv = ones(n,1);
    nnzpv = spones(mat);
else
    A = spones(mat);
    if (size(A,1) ~= m)
        error('in rowwise unsym size(A,1) ~= size(mat,1))');
    end
    if (size(A,2) ~= n)
        error('in rowwise unsym size(A,2) ~= size(mat,2))');
    end
    if (n > 1)
        rowsz = A*ones(n,1);
    else
        rowsz = full(A);
    end
    if (sum(rowsz) ~= nnz(mat))
        error('spones yield different numnnz??? Please let Bora know.\n');
    end

    [pv,ptime] = Partition(partitioner, A, m, K, 1, rowsz);
    outpv = pv + 1;
    [rr, cc, pno] = find(mat);
    pno(1:length(rr)) = outpv(rr);

    nnzpv=sparse(rr,cc, pno, m, n);

    inpv = zeros(n, 1);
    pwin = zeros(K,1);
    for j=1:n
        inColj = find(mat(:, j));
        topart = assignToOne(outpv(inColj), pwin);
        pwin(topart) = pwin(topart)+1;
        inpv(j) = topart;
    end
    if (nnz(nnzpv)~=nnz(mat))
        error('in RWU nnz(nnzpv)~=nnz(mat)\n');
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [nnzpv,outpv,inpv,ptime] = colwiseSymPart(mat,K, partitioner)
ptime = 0;
[m,n] = size(mat);
if (K==1)
    outpv = ones(m,1);
    inpv = ones(n,1);
    nnzpv = spones(mat);
else
    A = spones(mat);
    if (m > 1)
        colsz = (A')*ones(m,1);
    else
        colsz = full(A');
    end
    In = speye(m);
    A = (A | In);
    A = A';%transpose it

    [pv,ptime] = Partition(partitioner, A, n, K, 1, colsz);
    inpv = pv + 1;
    outpv = inpv;
    [rr, cc, pno] = find(mat);

    pno(1:length(cc)) = inpv(cc);

    nnzpv=sparse(rr,cc, pno, m, n);
    if (nnz(nnzpv)~=nnz(mat))
        error('in CWS nnz(nnzpv)~=nnz(mat)\n');
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [nnzpv,outpv,inpv,ptime] = colwiseUnsymPart(mat,K, partitioner)
ptime = 0;
[m,n] = size(mat);
if (K==1)
    outpv = ones(m,1);
    inpv = ones(n,1);
    nnzpv = spones(mat);
else
    A = spones(mat);
    A = A';%transpose it
    if (m > 1)
        colsz = A*ones(m,1);
    else
        colsz = full(A);
    end
    [pv,ptime] = Partition(partitioner, A, m, K, 1, colsz);
    inpv = pv + 1;
    [rr, cc, pno] = find(mat);

    pno(1:length(cc)) = inpv(cc);

    nnzpv=sparse(rr,cc, pno, m, n);
    if (nnz(nnzpv)~=nnz(mat))
        error('what is that CWU\n');
    end
    outpv = zeros(m,1);
    pwout = zeros(K,1);
    for i=1:m
        inRowi = find(A(:, i));
        topart = assignToOne(inpv(inRowi), pwout);
        pwout(topart) = pwout(topart)+1;
        outpv(i) = topart;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [nnzpv,outpv,inpv,ptime] = fineGrainSymPart(mat, K, partitioner)
ptime = 0;
[m,n] = size(mat);
if (K==1)
    outpv = ones(m,1);
    inpv = ones(n,1);
    nnzpv = spones(mat);
else
    dorg = spdiags(spones(mat), 0);   % with zeros
    dindex = find(dorg == 0);
    missingDiags = length(dindex);
    nnzmat = nnz(mat);
    numNnz = nnzmat + missingDiags;

    [rr, cc] = find(mat);
    rrext = [rr; dindex];
    ccext = [cc; dindex];
    hrr = [1:numNnz, 1:numNnz]';

    hcc = zeros(2*numNnz, 1);
    hcc(1:numNnz) = rrext;
    hcc(numNnz+1: 2*numNnz) = ccext + m;

    tmp = ones(2*numNnz, 1);
    cw = zeros(numNnz,1);
    cw(1: nnzmat) =1;
    cw(nnzmat+1:numNnz) = 0;

    hyp = sparse(hrr,hcc,tmp, numNnz, m+n);

    [pv,ptime] = PaToH(hyp, K, 1, cw);   %% UVCUVC TODO change partitoner
    pno = pv + 1;
    nnzpv=sparse(rrext,ccext, pno, m, n);
    inpv = spdiags(nnzpv, 0);
    outpv = inpv;
    %
    %
    %correct diags from nnzpv added for the sake of model
    dorg = spdiags(spones(mat), 0);   % with zeros
    dmod = spdiags(nnzpv , 0);        % no zeros
    dmod = dmod .* dorg;
    nnzpv = spdiags(dmod,0,nnzpv);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [nnzpv,outpv,inpv,ptime] = fineGrainUnsymPart(mat, K, partitioner)
ptime = 0;
[m,n] = size(mat);
if (K==1)
    outpv = ones(m,1);
    inpv = ones(n,1);
    nnzpv = spones(mat);
else
    A = spones(mat);
    numNnz = nnz(A);
    [rr, cc] = find(A);
    hrr = [1:numNnz, 1:numNnz]';

    hcc = zeros(2*numNnz, 1);
    hcc(1:numNnz) = rr;
    hcc(numNnz+1: 2*numNnz) = cc + m;

    tmp = ones(2*numNnz, 1);
    hyp = sparse(hrr,hcc,tmp, numNnz, m+n);

    cw = ones(numNnz,1);

    [pv,ptime] = PaToH(hyp, K, 1, cw);  %% UVCUVC TODO change partitoner

    pno = pv + 1;
    nnzpv=sparse(rr,cc, pno, m, n);
    [outpv, inpv] = PaToHNaiveVectorPart(nnzpv, 0);
end
for i=1:m
    if (outpv(i) <= 0 || outpv(i)>K)
        error('in PaToHMatrixPart outpv(%d)=%d for K=%d\n',i, outpv(i), K);
    end
end
for j=1:n
    if (inpv(j) <= 0 || inpv(j)>K)
        error('in PaToHMatrixPart inpv(%d)=%d for K=%d\n',i, inpv(i), K);
    end
end

if (nnz(nnzpv)~=nnz(mat))
    error('in fg unsym, nnz(nnzpv)~=nnz(mat)');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [nnzpv,outpv,inpv,ptime] = jaggedSymPart(mat,k1,k2, partitioner)
ptime = 0;
[m,n] = size(mat);
timerow = 0;
timecol = 0;
if (k1*k2==1)
    outpv = ones(m,1);
    inpv = ones(n,1);
    nnzpv = spones(mat);
else

    A = spones(mat);
    In = speye(m);
    A = spones(A | In);
    %%nnzpv = -spones(A);
    rindnnzpv = zeros(nnz(A),1);
    cindnnzpv = zeros(nnz(A),1);
    pindnnzpv = zeros(nnz(A),1);
    whereTo = 1;
    if (k1*k2 == 2)
        [nnzpv, outpv, inpv, timerow] = rowwiseSymPart(mat, 2, partitioner);
    else
        [dbgnnzpv, orow, dbgcol, timerow] = rowwiseSymPart(mat, k1, partitioner);
        % PaToHSpy(k1, orow, dbgcol, dbgnnzpv, '', 'x', 1)

        timecol = 0;
        for submat = 1:k1
            rowIndices = find(orow == submat);
            matsub = A(rowIndices,1:n);
            if (size(matsub,2)~= n)
                error('where are my 1-cols\n');
            end

            [nnzsub, discard1, discard2, timesub] = colwiseUnsymPart(matsub, k2, partitioner);
            timecol = timecol + timesub;
            [ii,jj,pp] = find(nnzsub);
            pp = pp - 1;
            pp = pp * k1 + submat;

            for ind=1:length(ii)
                orr = rowIndices(ii(ind));
                orc = jj(ind);
                rindnnzpv(whereTo) = orr;
                cindnnzpv(whereTo) = orc;
                pindnnzpv(whereTo) = pp(ind);
                whereTo = whereTo+1;
            end
        end
        nnzpv  = sparse(rindnnzpv,  cindnnzpv,pindnnzpv, m, n);
        inpv = spdiags(nnzpv, 0);
        outpv = inpv;
        dorg = spdiags(spones(mat), 0);   % with zeros
        dmod = spdiags(nnzpv , 0);        % no zeros
        dmod = dmod .* dorg;
        nnzpv = spdiags(dmod,0,nnzpv);
        %   PaToHSpy(k1*k2, outpv, inpv, nnzpv, '', 'x', 1)
    end
end
ptime = timerow + timecol;
if (nnz(nnzpv)~=nnz(mat))
    error('in jg sym, nnz(nnzpv)~=nnz(mat)');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [nnzpv,outpv,inpv,ptime] = jaggedUnsymPart(mat,k1,k2, partitioner)
ptime = 0;
[m,n] = size(mat);
timerow  = 0;
timecol = 0;
if (k1*k2==1)
    outpv = ones(m,1);
    inpv = ones(n,1);
    nnzpv = spones(mat);
else
    A = spones(mat);

    outpv = zeros(size(mat,1),1);
    inpv = zeros(size(mat,2),1);

    rindnnzpv = zeros(nnz(A),1);
    cindnnzpv = zeros(nnz(A),1);
    pindnnzpv = zeros(nnz(A),1);
    whereTo = 1;
    if (k1*k2 == 2)
        [nnzpv, outpv, inpv, timerow] =...
            rowwiseUnsymPart(mat, 2, partitioner);
    else
        [discard1, orow, discard2, timerow] = ...
            rowwiseUnsymPart(mat, k1, partitioner);

        timecol = 0;
        for submat = 1:k1
            rowIndices = find(orow == submat);
            matsub = mat(rowIndices,1:n);
            if (size(matsub,2)~= n)
                error('where are my 1-cols\n');
            end

            [nnzsub,discard1,discard2,timesub] = ...
                colwiseUnsymPart(matsub, k2, partitioner);
            timecol = timecol + timesub;
            [ii,jj,pp] = find(nnzsub);
            pp = pp + (submat-1) * k2;
            for ind=1:length(ii)
                orr = rowIndices(ii(ind));
                orc = jj(ind);
                rindnnzpv(whereTo) = orr;
                cindnnzpv(whereTo) = orc;
                pindnnzpv(whereTo) = pp(ind);
                whereTo = whereTo+1;
            end
        end
        nnzpv = sparse(rindnnzpv,cindnnzpv,pindnnzpv,size(A,1),size(A,2));
        [outpv, inpv] = PaToHNaiveVectorPart(nnzpv, 0);
    end
end
for i=1:m
    if (outpv(i) <= 0 || outpv(i)>k1*k2)
        error('in PaToHMatrixPart outpv(%d)=%d for K=%d\n',i, outpv(i), k1*k2);
    end
end
for j=1:n
    if (inpv(j) <= 0 || inpv(j)>k1*k2)
        error('in PaToHMatrixPart inpv(%d)=%d for K=%d\n',i, inpv(i), k1*k2);
    end
end

ptime = timerow + timecol;
if (nnz(nnzpv)~=nnz(mat))
    error('in jg unsym, nnz(nnzpv)~=nnz(mat)');
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [nnzpv,outpv,inpv,ptime] = chkbrdSymPart(mat,k1,k2, partitioner)
[m,n] = size(mat);
timerow = 0;
timecol = 0;
if (k1*k2==1)
    outpv = ones(m,1);
    inpv = ones(n,1);
    nnzpv = spones(mat);
else
    A = spones(mat);
    In = speye(m);
    A = (A | In);
    if (partitioner==0)
        [nnzrow, orow, discard, timerow] = ...
            rowwiseSymPart(mat, k1, partitioner);
        %            PaToHSpy(k1, orow, discard, nnzrow, '', 'x', 1)

        %%%%%%%%%%%%%%%%%%%
        %MultiConstraint!!!
        %
        nconst = k1;
        cw = zeros(nconst * n , 1);
        for j=1:size(nnzrow,2)
            cwvec = zeros(k1,1);
            [discard1, discard2, pp] = find(nnzrow(:, j));
            for pind=1:length(pp)
                tpno = pp(pind);
                cwvec(tpno) = cwvec(tpno)+1;
            end
            offstr = (j-1)*k1 + 1;
            offend = j*k1;
            cw(offstr:offend) = cwvec;
        end
        if (sum(cw) ~= nnz(mat))
            error('in chkbrd sym sum(cw)%d~=nnz(mat)%d',sum(cw),nnz(mat));
        end
        [ocol,timecol] = PaToH(A', k2, nconst, cw);
        ocol = ocol+1;
    elseif (partitioner == 1)
        orow = randomPart(m, k1) + 1;
        ocol = randomPart(n, k2) + 1;
    elseif (partitioner == 2)
        orow = blockPart(m, k1) + 1;
        ocol = blockPart(n, k2) + 1;
    elseif (partitioner == 2)
        error('partitioner=%d not implemented yet', partitioner);
    elseif (partitioner == 4)
        orow = roundRobinPart(m, k1) + 1;
        ocol = roundRobinPart(n, k2) + 1;
    else
        error('unknow partitioner=%d', partitioner);
    end

    %
    %
    %%%%%%%%%%%%%%%%%%%
    [rr,cc,discard] = find(A);% With nonzero diags
    pnos = zeros(length(rr),1);
    for i=1:length(rr)
        rno = rr(i);
        cno = cc(i);
        % in node addresses most significat bits are x
        %         coordinate in processor mesh.
        p = orow(rno) + k1 * (ocol(cno)-1);
        pnos(i) = p;
    end
    nnzpv = sparse(rr,cc,pnos, m, n);
    inpv = spdiags(nnzpv, 0);
    outpv = inpv;
    dorg = spdiags(spones(mat), 0);   % with zeros
    dmod = spdiags(nnzpv , 0);        % no zeros
    dmod = dmod .* dorg;
    nnzpv = spdiags(dmod,0,nnzpv);
    %        PaToHSpy(k1*k2, outpv, inpv, nnzpv, '', 'x', 1)
end

ptime = timerow + timecol;
if (nnz(nnzpv)~=nnz(mat))
    error('in chkbrd sym, nnz(nnzpv)~=nnz(mat)');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [nnzpv,outpv,inpv,ptime] = chkbrdUnsymPart(mat,k1,k2, partitioner)
[m,n] = size(mat);
timerow = 0;
timecol = 0;
if (k1*k2==1)
    outpv = ones(m,1);
    inpv = ones(n,1);
    nnzpv = spones(mat);
else
    outpv = zeros(m,1);
    inpv = zeros(n,1);
    if (partitioner==0)
        [nnzrow, orow, discard, timerow] = ...
            rowwiseUnsymPart(mat, k1, partitioner);
        %%%%%%%%%%%%%%%%%%%
        %MultiConstraint!!!
        nconst = k1;
        cw = zeros(nconst * n , 1);
        for j=1:size(nnzrow,2)
            cwvec = zeros(k1,1);
            [discard1, discard2, pp] = find(nnzrow(:, j));
            for l=1:length(pp)
                cwvec(pp(l)) = cwvec(pp(l))+1;
            end
            offstr = (j-1)*k1 + 1;
            offend = j*k1;
            cw(offstr:offend) = cwvec;
        end
        A = spones(mat)';
        [ocol,timecol] = PaToH(A, k2, nconst, cw); %% UVCUVC TODO change partitoner
        ocol = ocol + 1;
    elseif (partitioner == 1)
        orow = randomPart(m, k1) + 1;
        ocol = randomPart(n, k2) + 1;
    elseif (partitioner == 2)
        orow = blockPart(m, k1) + 1;
        ocol = blockPart(n, k2) + 1;
    elseif (partitioner == 2)
        error('partitioner=%d not implemented yet', partitioner);
    elseif (partitioner == 4)
        orow = roundRobinPart(m, k1) + 1;
        ocol = roundRobinPart(n, k2) + 1;
    else
        error('unknow partitioner=%d', partitioner);
    end

    %
    %%%%%%%%%%%%%%%%%%%
    [rr,cc,discard] = find(mat);
    pnos = zeros(length(rr),1);
    for i=1:length(rr)
        rno = rr(i);
        cno = cc(i);
        p = (orow(rno)-1) * k2 + ocol(cno);
        pnos(i) = p;
    end
    nnzpv = sparse(rr,cc,pnos, m, n);
    [outpv, inpv] = PaToHNaiveVectorPart(nnzpv, 0);
end

for i=1:m
    if (outpv(i) <= 0 || outpv(i)>k1*k2)
        error('in PaToHMatrixPart outpv(%d)=%d for K=%d\n',i, outpv(i), k1*k2);
    end
end
for j=1:n
    if (inpv(j) <= 0 || inpv(j)>k1*k2)
        error('in PaToHMatrixPart inpv(%d)=%d for K=%d\n',i, inpv(i), k2*k2);
    end
end

ptime  = timerow + timecol;

if (nnz(nnzpv)~=nnz(mat))
    error('in chkbrd unsym, nnz(nnzpv)~=nnz(mat)');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function pno=assignToOne(plist, pwght)
if (isempty(plist))
    [discard, topart] = min(pwght);
    pno = topart;
else
    [discard, whichpart] = min(pwght(plist));
    pno = plist(whichpart);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function  isit = PaToHisSymmetricPart(dim)
% isit = PaToHisSymmetricPart(dim)
% returns 1 if dim='RWS' or 'CWS' or 'FGS' or 'CHS' or 'JLS'.
% otherwise returns 0.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Bora Ucar: May, July 2007.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ((strcmpi(dim,'RWS') == 1) ||...
        (strcmpi(dim, 'CWS') == 1)||...
        (strcmpi(dim, 'FGS') == 1)||...
        (strcmpi(dim, 'CHS') == 1)||...
        (strcmpi(dim, 'JLS') == 1))
    isit=1;
else
    isit=0;
end
